%% Init file for Simulink model power_system_11.slx
% 
clear all;

%% System wide settings
N = 11; % Number of control areas
L = 2500; % [MW] Power base value
sim_time = 10;

%% Control Center
% Sample time for generating new AGC set-points
sample_time_frr_a = 10;

% Extract FRR-A bids to matrices 
bidListUp = xlsread('bids/FRR-A_bids_all_up');
bidListDown = xlsread('bids/FRR-A_bids_all_down');
[Q_min_up, Q_max_up, C_up] = generateBidMat(bidListUp, N);
[Q_min_down, Q_max_down, C_down] = generateBidMat(bidListDown, N);

% Optimization paramters
flowCost = 0.001;

%% Parameters for control areas
% parameters = [T_i T_w H EMPTY R beta EMPTY Ki_sec]
R = 0.25;
Kp = 0;
Tg = 5;
T_agc = 200;
beta = 0.26;

test_param = [Tg 1.5 3.5 0 R beta Kp -1/T_agc];
param_NO1 = [Tg 1.5 3.5 0 R beta Kp -1/T_agc];
param_NO2 = [Tg 1.5 3.5 0 0.24 0.26 Kp -1/T_agc];
param_NO3 = [Tg 1.5 3.5 0 0.25 0.25 Kp -1/T_agc];
param_NO4 = [Tg 1.5 3.5 0 0.35 0.34 Kp -1/T_agc];
param_NO5 = [Tg 1.5 3.5 0 0.30 0.26 Kp -1/T_agc];

param_SE1 = [Tg 1.5 3.5 0 0.26 0.27 Kp -1/T_agc];
param_SE2 = [Tg 1.5 3.5 0 0.25 0.26 Kp -1/T_agc];
param_SE3 = [Tg 1.5 3.5 0 0.23 0.24 Kp -1/T_agc];
param_SE4 = [Tg 1.5 3.5 0 0.22 0.23 Kp -1/T_agc];

param_FI = [Tg 1.5 3.5 0 0.22 0.25 Kp -1/T_agc];
param_DK2 = [Tg 1.5 3.5 0 0.18 0.19 Kp -1/T_agc];

D_tieline = 0.05;
%% Tieline Parameters
% TIE-LINE CONNECTION MATRIX
% [branches, nodes]
A = [   1   -1  0   0   0   0   0   0   0   0   0;      % B1
        1   0   -1  0   0   0   0   0   0   0   0;      % B2
        1   0   0   0   -1  0   0   0   0   0   0;      % B3
        1   0   0   0   0   0   0   -1  0   0   0;      % B4
        0   1   0   0   -1  0   0   0   0   0   0;      % B5
        0   0   1   0   -1  0   0   0   0   0   0;      % B6
        0   0   1   -1  0   0   0   0   0   0   0;      % B7
        0   0   1   0   0   0   -1  0   0   0   0;      % B8
        0   0   0   1   0   -1  0   0   0   0   0;      % B9
        0   0   0   1   0   0   -1  0   0   0   0;      % B10
        0   0   0   1   0   0   0   0   0   -1  0;      % B11
        0   0   0   0   0   1   -1  0   0   0   0;      % B12
        0   0   0   0   0   1   0   0   0   -1  0;      % B13
        0   0   0   0   0   0   1   -1  0   0   0;      % B14
        0   0   0   0   0   0   0   1   -1  0   0;      % B15
        0   0   0   0   0   0   0   0   1   0   -1];    % B16

% Tie-line connection matrix #2
branches = [1 2; 1 3; 1 5; 1 8; 2 5; 3 5; 3 4; 3 7; 4 6; 4 7; 4 10; 6 7; 6 10; 7 8; 8 9; 9 11];
    
% Capacity
minFlowVector = [-1000 -1000 -1000 -1000 -1000 -1000 -1000 -1000 ...
                 -1000 -1000 -1000 -1000 -1000 -1000 -1000 -1000];
maxFlowVector = [1000 1000 1000 1000 1000 1000 1000 1000 ...
                 1000 1000 1000 1000 1000 1000 1000 1000];

%% Tieline Admittance


Y = [3.00 3.10 2.90 3.15 2.98 2.95 3.16 2.97 3.01 2.87 3.22 3.14 2.89 2.95 2.85 3.01]; 
% Model of admittance with error
Y_m = ones(1,16) * 3;

% Admittance matrix must have zeros on diagonal
Ymat = ...
[0    Y(1)  Y(2)  0     Y(3)  0     0     Y(4)  0     0     0;
Y(1)  0     0     0     Y(5)  0     0     0     0     0     0;
Y(2)  0     0     Y(7)  Y(6)  0     Y(8)  0     0     0     0;
0     0     Y(7)  0     0     Y(9)  Y(10) 0     0     Y(11) 0;
Y(3)  Y(5)  Y(6)  0     0     0     0     0     0     0     0;
0     0     0     Y(9)  0     0     Y(12) 0     0     Y(13) 0;
0     0     Y(8)  Y(10) 0     Y(12) 0     Y(14) 0     0     0;
Y(4)  0     0     0     0     0     Y(14) 0     Y(15) 0     0;
0     0     0     0     0     0     0     Y(15) 0     0     Y(16);
0     0     0     Y(11) 0     Y(13) 0     0     0     0     0;
0     0     0     0     0     0     0     0     Y(16) 0     0];

Ymat_model = ...
        [0       Y_m(1)  Y_m(2)  0       Y_m(3)  0       0       Y_m(4)  0       0       0;
        Y_m(1)  0       0       0       Y_m(5)  0       0       0       0       0       0;
        Y_m(2)  0       0       Y_m(7)  Y_m(6)  0       Y_m(8)  0       0       0       0;
        0       0       Y_m(7)  0       0       Y_m(9)  Y_m(10) 0       0       Y_m(11) 0;
        Y_m(3)  Y_m(5)  Y_m(6)  0       0       0       0       0       0       0       0;
        0       0       0       Y_m(9)  0       0       Y_m(12) 0       0       Y_m(13) 0;
        0       0       Y_m(8)  Y_m(10) 0       Y_m(12) 0       Y_m(14) 0       0       0;
        Y_m(4)  0       0       0       0       0       Y_m(14) 0       Y_m(15) 0       0;
        0       0       0       0       0       0       0       Y_m(15) 0       0       Y_m(16);
        0       0       0       Y_m(11) 0       Y_m(13) 0       0       0       0       0;
        0       0       0       0       0       0       0       0       Y_m(16) 0       0];
    

    
% Generate B_prime matrix 
B_prime = (diag(sum(Ymat_model,2)') - Ymat_model);  

% Negative suseptence 
D = diag(Y_m);      

%% Simulate
% sim power_system_v2_11;

%% Plot
%plot_11;
%plot_11_FRRA;
%plot_tielines;

%plot_area('NO1.mat', L, 1);  
%plot_area('NO2.mat', L, 2);
%plot_area('NO3.mat', L, 3);
%plot_area('NO4.mat', L, 4);
%plot_area('NO5.mat', L, 5);
%plot_area('SE1.mat', L, 6);  
%plot_area('SE2.mat', L, 7);
%plot_area('SE3.mat', L, 8);
%plot_area('SE4.mat', L, 9);
%plot_area('FI.mat', L, 10);
%plot_area('DK2.mat', L, 11);

